@@ -1,11 +1,11 @@
 /*
  *	sslDecode.c
- *	Release $Name: MATRIXSSL_1_8_5_OPEN $
+ *	Release $Name: MATRIXSSL_1_8_8_OPEN $
  *
  *	Secure Sockets Layer message decoding
  */
 /*
- *	Copyright (c) PeerSec Networks, 2002-2008. All Rights Reserved.
+ *	Copyright (c) PeerSec Networks, 2002-2009. All Rights Reserved.
  *	The latest version of this code is available at http://www.matrixssl.org
  *
  *	This software is open source; you can redistribute it and/or modify
@@ -36,6 +36,8 @@
 #define SSL_MAX_IGNORED_MESSAGE_COUNT	1024
 
 static int32 parseSSLHandshake(ssl_t *ssl, char *inbuf, int32 len);
+static int32 parseSingleCert(ssl_t *ssl, unsigned char *c, unsigned char *end, 
+						   int32 certLen);
 
 /******************************************************************************/
 /*
@@ -121,9 +123,9 @@ decodeMore:
 	past ClientHello) and the length must be < 16K and > 0
 */
 	if (ssl->rec.type != SSL_RECORD_TYPE_CHANGE_CIPHER_SPEC &&
-		ssl->rec.type != SSL_RECORD_TYPE_ALERT &&
-		ssl->rec.type != SSL_RECORD_TYPE_HANDSHAKE &&
-		ssl->rec.type != SSL_RECORD_TYPE_APPLICATION_DATA) {
+			ssl->rec.type != SSL_RECORD_TYPE_ALERT &&
+			ssl->rec.type != SSL_RECORD_TYPE_HANDSHAKE &&
+			ssl->rec.type != SSL_RECORD_TYPE_APPLICATION_DATA) {
 		ssl->err = SSL_ALERT_UNEXPECTED_MESSAGE;
 		matrixIntDebugMsg("Record header type not valid: %d\n", ssl->rec.type);
 		goto encodeResponse;
@@ -300,6 +302,7 @@ decodeMore:
 			matrixStrDebugMsg("Invalid value for CipherSpec\n", NULL);
 			goto encodeResponse;
 		}
+		
 /*
 		If we're expecting finished, then this is the right place to get
 		this record.  It is really part of the handshake but it has its
@@ -495,14 +498,16 @@ static int32 parseSSLHandshake(ssl_t *ssl, char *inbuf, int32 len)
 
 #ifdef USE_CLIENT_SIDE_SSL
 	int32			sessionIdLen, certMatch, certTypeLen;
+	sslCert_t		*subjectCert;
 	int32			valid, certLen, certChainLen, anonCheck;
-	sslRsaCert_t	*cert, *currentCert, *subjectCert;
+	sslCert_t		*cert, *currentCert;
 #endif /* USE_CLIENT_SIDE_SSL */
 
 	rc = SSL_SUCCESS;
 	c = (unsigned char*)inbuf;
 	end = (unsigned char*)(inbuf + len);
 
+
 parseHandshake:
 	if (end - c < 1) {
 		ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
@@ -510,6 +515,20 @@ parseHandshake:
 		return SSL_ERROR;
 	}
 	hsType = *c; c++;
+
+#ifndef ALLOW_SERVER_REHANDSHAKES
+/*
+	Disables server renegotiation.  This is in response to the HTTPS
+	flaws discovered by Marsh Ray in which a man-in-the-middle may take
+	advantage of the "authentication gap" in the SSL renegotiation protocol	
+*/
+	if (hsType == SSL_HS_CLIENT_HELLO && ssl->hsState == SSL_HS_DONE) {
+		ssl->err = SSL_ALERT_UNEXPECTED_MESSAGE;
+		matrixStrDebugMsg("Server rehandshaking is disabled\n", NULL);
+		return SSL_ERROR;
+	}
+#endif /* ALLOW_SERVER_REHANDSHAKES */
+
 /*
 	hsType is the received handshake type and ssl->hsState is the expected
 	handshake type.  If it doesn't match, there are some possible cases
@@ -531,24 +550,28 @@ parseHandshake:
 */
 			ssl->flags |= SSL_FLAGS_CLIENT_AUTH;
 			ssl->hsState = SSL_HS_CERTIFICATE_REQUEST;
-		} else {
-/*
-			The final possible mismatch allowed is for a HELLO_REQEST message.
-			Indicates a rehandshake initiated from the server.
-*/
-			if ((hsType == SSL_HS_HELLO_REQUEST) &&
-					(ssl->hsState == SSL_HS_DONE) &&
-					!(ssl->flags & SSL_FLAGS_SERVER)) {
-				sslResetContext(ssl);
-				ssl->hsState = hsType;
-			} else {
-				ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
-				matrixIntDebugMsg("Invalid type of handshake message: %d\n",
-					hsType);
-				return SSL_ERROR;
-			}
+			goto hsStateDetermined;
 		}
+/*
+		Another possible mismatch allowed is for a HELLO_REQEST message.
+		Indicates a rehandshake initiated from the server.
+*/
+		if ((hsType == SSL_HS_HELLO_REQUEST) &&
+				(ssl->hsState == SSL_HS_DONE) &&
+				!(ssl->flags & SSL_FLAGS_SERVER)) {
+			sslResetContext(ssl);
+			ssl->hsState = hsType;
+			goto hsStateDetermined;
+		}
+
+
+
+		ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
+		matrixIntDebugMsg("Invalid type of handshake message: %d\n", hsType);
+		return SSL_ERROR;
 	}
+	
+hsStateDetermined:	
 	if (hsType == SSL_HS_CLIENT_HELLO) { 
 		sslInitHSHash(ssl);
 		if (ssl->hsState == SSL_HS_DONE) {
@@ -592,8 +615,8 @@ parseHandshake:
 			matrixStrDebugMsg("Invalid handshake length\n", NULL);
 			return SSL_ERROR;
 		}
-		sslUpdateHSHash(ssl, c - ssl->hshakeHeadLen,
-			hsLen + ssl->hshakeHeadLen);
+			sslUpdateHSHash(ssl, c - ssl->hshakeHeadLen,
+				hsLen + ssl->hshakeHeadLen);
 
 	} else if (ssl->rec.majVer == SSL2_MAJ_VER) {
 /*
@@ -745,9 +768,7 @@ parseHandshake:
 			}
 
 /*
-			Decode the compression parameters.  Always length one (first byte)
-			and value 0 (second byte).  There are no compression schemes defined
-			for SSLv3
+			Bypass the compression parameters.  Only supporting mandatory NULL
 */
 			if (end - c < 1) {
 				ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
@@ -761,38 +782,6 @@ parseHandshake:
 				return SSL_ERROR;
 			}
 			c += extLen;
-
-			if (ssl->reqMinVer == SSL3_MIN_VER && extLen != 1) {
-				ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
-				matrixStrDebugMsg("Invalid compression header\n", NULL);
-				return SSL_ERROR;
-			}
-/*
-			This might be TLS.  If so, there could be extension data
-			to parse here:  Two byte length and extension info.
-			http://www.faqs.org/rfcs/rfc3546.html
-*/
-			if (c != end) {
-				if (end - c < 2) {
-					ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
-					matrixStrDebugMsg("Invalid extension header len\n", NULL);
-					return SSL_ERROR;
-				}
-				extLen = *c << 8; c++;
-				extLen += *c; c++;
-				if (end - c < extLen) {
-					ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
-					matrixStrDebugMsg("Invalid extension header len\n", NULL);
-					return SSL_ERROR;
-				}
-/*
-				At this point, c points to the first byte of the extension field
-				and extLen is the number of extension bytes.  Parsing of 
-				individual extension values would happen here, but for now we
-				just skip over all extensions, ignoring them.
-*/
-				c += extLen;
-			}
 		} else {
 /*
 			Parse a SSLv2 ClientHello message.  The same information is 
@@ -887,6 +876,7 @@ parseHandshake:
 			return SSL_ERROR;
 		}
 
+
 /*
 		If we're resuming a handshake, then the next handshake message we
 		expect is the finished message.  Otherwise we do the full handshake.
@@ -923,48 +913,50 @@ parseHandshake:
 		pubKeyLen = hsLen;
 
 
+/*
+				Now have a handshake pool to allocate the premaster storage
+*/
+				ssl->sec.premasterSize = SSL_HS_RSA_PREMASTER_SIZE;
+				ssl->sec.premaster = psMalloc(ssl->hsPool,
+					SSL_HS_RSA_PREMASTER_SIZE);
+
+				if (ssl->decryptPriv(ssl->hsPool, ssl->keys->cert.privKey, c,
+						pubKeyLen, ssl->sec.premaster, ssl->sec.premasterSize)
+						!= ssl->sec.premasterSize) {
+					ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
+					matrixStrDebugMsg("Failed to decrypt premaster\n", NULL);
+					return SSL_ERROR;
+				}
 
 /*
-			Now have a handshake pool to allocate the premaster storage
+				The first two bytes of the decrypted message should be the
+				client's requested version number (which may not be the same
+				as the final negotiated version). The other 46 bytes -
+				pure random!
+			
+				SECURITY - 
+				Some SSL clients (Including Microsoft IE 6.0) incorrectly set
+				the first two bytes to the negotiated version rather than the
+				requested version.  This is known in OpenSSL as the
+				SSL_OP_TLS_ROLLBACK_BUG. We allow this to slide only if we
+				don't support TLS, TLS was requested and the negotiated
+				versions match.
 */
-			ssl->sec.premasterSize = SSL_HS_RSA_PREMASTER_SIZE;
-			ssl->sec.premaster = psMalloc(ssl->hsPool,
-				SSL_HS_RSA_PREMASTER_SIZE);
-
-			if (ssl->decryptPriv(ssl->hsPool, ssl->keys->cert.privKey, c,
-					pubKeyLen, ssl->sec.premaster, ssl->sec.premasterSize) != 
-					ssl->sec.premasterSize) {
-				ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
-				matrixStrDebugMsg("Failed to decrypt premaster\n", NULL);
-				return SSL_ERROR;
-			}
-
-/*
-			The first two bytes of the decrypted message should be the client's 
-			requested version number (which may not be the same as the final 
-			negotiated version). The other 46 bytes - pure random!
-			SECURITY - 
-			Some SSL clients (Including Microsoft IE 6.0) incorrectly set the
-			first two bytes to the negotiated version rather than the requested
-			version.  This is known in OpenSSL as the SSL_OP_TLS_ROLLBACK_BUG
-			We allow this to slide only if we don't support TLS, TLS was
-			requested and the negotiated versions match.
-*/
-			if (*ssl->sec.premaster != ssl->reqMajVer) {
-				ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
-				matrixStrDebugMsg("Incorrect version in ClientKeyExchange\n",
-					NULL);
-				return SSL_ERROR;
-			}
-			if (*(ssl->sec.premaster + 1) != ssl->reqMinVer) {
-				if (ssl->reqMinVer < TLS_MIN_VER ||
-						*(ssl->sec.premaster + 1) != ssl->minVer) {
+				if (*ssl->sec.premaster != ssl->reqMajVer) {
 					ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
 					matrixStrDebugMsg("Incorrect version in ClientKeyExchange\n",
 						NULL);
 					return SSL_ERROR;
 				}
-			}
+				if (*(ssl->sec.premaster + 1) != ssl->reqMinVer) {
+					if (ssl->reqMinVer < TLS_MIN_VER ||
+							*(ssl->sec.premaster + 1) != ssl->minVer) {
+						ssl->err = SSL_ALERT_ILLEGAL_PARAMETER;
+						matrixStrDebugMsg("Incorrect version in ClientKeyExchange\n",
+							NULL);
+						return SSL_ERROR;
+					}
+				}
 
 /*
 		Now that we've got the premaster secret, derive the various
@@ -1087,7 +1079,7 @@ parseHandshake:
 			matrixIntDebugMsg("Unsupported ssl version: %d\n", ssl->reqMajVer);
 			return SSL_ERROR;
 		}
-		
+
 /*
 		Next is a 32 bytes of random data for key generation
 		and a single byte with the session ID length
@@ -1165,6 +1157,7 @@ parseHandshake:
 			}
 		}
 
+
 /*
 		Decode the compression parameters.  Always zero.
 		There are no compression schemes defined for SSLv3
@@ -1180,6 +1173,7 @@ parseHandshake:
 		the Finished message.
 */
 		c++;
+		
 		if (ssl->flags & SSL_FLAGS_RESUMED) {
 			sslDeriveKeys(ssl);
 			ssl->hsState = SSL_HS_FINISHED;
@@ -1212,17 +1206,6 @@ parseHandshake:
 			return SSL_ERROR;
 		}
 
-		if (!(ssl->flags & SSL_FLAGS_SERVER)) {
-/*
-			As a client, we are aware we are in an RSA authentication state
-			at this point.  And now that we have a handshake pool, storage
-			for the known premaster secret can be allocated.
-*/
-			ssl->sec.premasterSize = SSL_HS_RSA_PREMASTER_SIZE;
-			ssl->sec.premaster = psMalloc(ssl->hsPool,
-				SSL_HS_RSA_PREMASTER_SIZE);
-		}
-
 		i = 0;
 		while (certChainLen > 0) {
 			certLen = *c << 16; c++;
@@ -1394,6 +1377,8 @@ parseHandshake:
 		ssl->err = SSL_ALERT_UNEXPECTED_MESSAGE;
 		return SSL_ERROR;
 	}
+	
+	
 /*
 	if we've got more data in the record, the sender has packed
 	multiple handshake messages in one record.  Parse the next one.
@@ -1405,4 +1390,3 @@ parseHandshake:
 }
 
 /******************************************************************************/
-
